class Commands:
    IDENTIFY_NEEDED = "identify_requested"
    SEND_WEBSOCKET_MESSAGE = "send_websocket_message_requested"
    CLIENT_RESET_NEEDED = "client_reset_needed"
    STOP_HB_WATCHDOG = "stop_heartbeat_watchdog"
    START_HB_WATCHDOG = "start_heartbeat_watchdog"
    RESTART_HB_WATCHDOG = "restart_heartbeat_watchdog"
    START_HB_LOOP = "start_heartbeat_loop"
    STOP_HB_LOOP = "stop_heartbeat_loop"
    START_LISTENER = "start_listener"
    STOP_LISTENER = "stop_listener"
    RECEIVE_WEBSOCKET_MESSAGE = "receive_websocket_message"
    SEND_ONE_HEARTBEAT = "send_one_heartbeat"
    HANDLE_EVENT = "handle_response_events"